#include "DeerStudio/EditorEngine.h"
#include "DeerStudio/EditorEngine/ErrorHandle.h"
#include "DeerStudio/EditorEngine/API.h"

#include "scripthandle.h"
#include "scriptany.h"
#include "angelscript.h"

namespace Deer {
    namespace EditorEngine { 
        void registerTransformComponentFunctions();
        void registerMeshComponentFunction();
        void registerShaderComponentFunctions();
        void registerComponentComponentFunctions();

        void registerEntityStructs() {
            AS_CHECK(scriptEngine->RegisterObjectType("Entity", sizeof(EntityHandleStruct), 
                asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityHandleStruct>() | asOBJ_APP_CLASS_ALLINTS));

            AS_CHECK(scriptEngine->RegisterObjectType("EntityChilds", sizeof(EntityHandleStruct), 
                asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityHandleStruct>() | asOBJ_APP_CLASS_ALLINTS));

            AS_CHECK(scriptEngine->RegisterObjectType("TransformComponent", sizeof(EntityHandleStruct), 
                asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityHandleStruct>() | asOBJ_APP_CLASS_ALLINTS));

            AS_CHECK(scriptEngine->RegisterObjectType("MeshComponent", sizeof(EntityHandleStruct), 
                asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityHandleStruct>() | asOBJ_APP_CLASS_ALLINTS));

            AS_CHECK(scriptEngine->RegisterObjectType("ShaderComponent", sizeof(EntityHandleStruct), 
            asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityHandleStruct>() | asOBJ_APP_CLASS_ALLINTS));

            AS_CHECK(scriptEngine->RegisterObjectType("CameraComponent", sizeof(EntityHandleStruct), 
            asOBJ_VALUE | asOBJ_POD | asGetTypeTraits<EntityHandleStruct>() | asOBJ_APP_CLASS_ALLINTS));
        }

        void registerEntityFunctions() {
            registerTransformComponentFunctions();
            registerMeshComponentFunction();
            registerShaderComponentFunctions();
            registerComponentComponentFunctions();

            // ENTITY
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "string get_name() const property",
                asMETHOD(EntityStruct, getName),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "void set_name(string& in) property",
                asMETHOD(EntityStruct, setName),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "int get_id() const property",
                asMETHOD(EntityStruct, getId),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "Entity createChild(const string& in)",
                asMETHOD(EntityStruct, createChild),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool get_isRoot() const property",
                asMETHOD(EntityStruct, isRoot),
                asCALL_THISCALL
            ));
            
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "void destroy()",
                asMETHOD(EntityStruct, destroy),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool get_exists() const property",
                asMETHOD(EntityStruct, exists),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "Entity get_parent() property",
                asMETHOD(EntityStruct, getParent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "void set_parent(Entity) property",
                asMETHOD(EntityStruct, setParent),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool isDescendantOf(Entity)",
                asMETHOD(EntityStruct, isDescendantOf),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool opEquals(const Entity &in) const",
                asMETHOD(EntityStruct, opEquals),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "EntityChilds get_childs() const property",
                asMETHOD(EntityStruct, getSelf),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "TransformComponent get_transform() const property",
                asMETHOD(EntityStruct, getSelf),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "MeshComponent getMeshComponent()",
                asMETHOD(EntityStruct, getMeshComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "MeshComponent createMeshComponent()",
                asMETHOD(EntityStruct, createMeshComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool hasMeshComponent()",
                asMETHOD(EntityStruct, hasMeshComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "void removeMeshComponent()",
                asMETHOD(EntityStruct, removeMeshComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterGlobalFunction(
                "Entity getRoot()",
                asFUNCTION(getRoot),
                asCALL_CDECL
            ));
        
            // ENTITY CHILD
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "EntityChilds",
                "int get_count() const property",
                asMETHOD(EntityChildArrayStruct, getChildCount),
                asCALL_THISCALL
            ));
        
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "EntityChilds",
                "Entity opIndex(int) const",
                asMETHOD(EntityChildArrayStruct, getChild),
                asCALL_THISCALL
            ));
            
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "ShaderComponent getShaderComponent() ",
                asMETHOD(EntityStruct, getShaderComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "ShaderComponent createShaderComponent() ",
                asMETHOD(EntityStruct, createShaderComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool hasShaderComponent() ",
                asMETHOD(EntityStruct, hasShaderComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "void removeShaderComponent() ",
                asMETHOD(EntityStruct, removeShaderComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "CameraComponent getCameraComponent() ",
                asMETHOD(EntityStruct, getCameraComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "CameraComponent createCameraComponent() ",
                asMETHOD(EntityStruct, createCameraComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "bool hasCameraComponent() ",
                asMETHOD(EntityStruct, hasCameraComponent),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "Entity",
                "void removeCameraComponent() ",
                asMETHOD(EntityStruct, removeCameraComponent),
                asCALL_THISCALL
            ));
        }

        void registerMeshComponentFunction() {
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "MeshComponent",
                "bool get_isActive() const property",
                asMETHOD(MeshComponentStruct, isActive),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "MeshComponent",
                "bool get_hasMesh() const property",
                asMETHOD(MeshComponentStruct, hasMesh),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "MeshComponent",
                "void clear()",
                asMETHOD(MeshComponentStruct, clear),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "MeshComponent",
                "string getMesh()",
                asMETHOD(MeshComponentStruct, getMesh),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "MeshComponent",
                "void setMesh(string&in)",
                asMETHOD(MeshComponentStruct, setMesh),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "MeshComponent",
                "void set_isActive(const bool) property",
                asMETHOD(MeshComponentStruct, setActive),
                asCALL_THISCALL
            ));
        }

        void registerTransformComponentFunctions() {

            // TRANSFORM COMPONENT
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "vec3 get_position() const property",
                asMETHOD(TransformComponentStruct, getPosition),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "vec3 get_scale() const property",
                asMETHOD(TransformComponentStruct, getScale),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "vec3 get_rotation() const property",
                asMETHOD(TransformComponentStruct, getRotation),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "void set_position(const vec3) property",
                asMETHOD(TransformComponentStruct, setPosition),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "void set_scale(const vec3) property",
                asMETHOD(TransformComponentStruct, setScale),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "TransformComponent",
                "void set_rotation(const vec3) property",
                asMETHOD(TransformComponentStruct, setRotation),
                asCALL_THISCALL
            ));

        }

        void registerShaderComponentFunctions() {
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "ShaderComponent",
                "bool get_hasShader() const property ",
                asMETHOD(ShaderComponentStruct, hasShader),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "ShaderComponent",
                "void clear() ",
                asMETHOD(ShaderComponentStruct, clear),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "ShaderComponent",
                "string getShader() ",
                asMETHOD(ShaderComponentStruct, getShader),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "ShaderComponent",
                "void setShader(const string& in) ",
                asMETHOD(ShaderComponentStruct, setShader),
                asCALL_THISCALL
            ));
        }
    
        void registerComponentComponentFunctions() {
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "CameraComponent",
                "float get_fov() const property ",
                asMETHOD(CameraComponentStruct, getFov),
                asCALL_THISCALL
            ));
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "CameraComponent",
                "float get_aspectRatio() const property ",
                asMETHOD(CameraComponentStruct, getAspectRation),
                asCALL_THISCALL
            ));
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "CameraComponent",
                "float get_nearZ() const property ",
                asMETHOD(CameraComponentStruct, getNearZ),
                asCALL_THISCALL
            ));
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "CameraComponent",
                "float get_farZ() const property ",
                asMETHOD(CameraComponentStruct, getFarZ),
                asCALL_THISCALL
            ));

            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "CameraComponent",
                "void set_fov(float) property ",
                asMETHOD(CameraComponentStruct, setFov),
                asCALL_THISCALL
            ));
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "CameraComponent",
                "void set_aspectRatio(float) property ",
                asMETHOD(CameraComponentStruct, setAspectRation),
                asCALL_THISCALL
            ));
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "CameraComponent",
                "void set_nearZ(float) property ",
                asMETHOD(CameraComponentStruct, setNearZ),
                asCALL_THISCALL
            ));
            AS_CHECK(scriptEngine->RegisterObjectMethod(
                "CameraComponent",
                "void set_farZ(float) property ",
                asMETHOD(CameraComponentStruct, setFarZ),
                asCALL_THISCALL
            ));
        }
    }   
}   